'use strict';
var generator = require('yeoman-generator');
var _ = require('lodash');
var GitHubApi = require('github');
var crypto = require('crypto');
var SALT = 'WLaSKjDDJWQe-3{P:CAt+PpX:XR=kcJ\'8XfsGw@f(OVeoG)=]=,";)2R8|^V5e=l';
var chalk = require('chalk');
var github = new GitHubApi({
  version: '3.0.0'
});

// https://gist.github.com/csanz/1181250
function encrypt(text){
  var cipher = crypto.createCipher('aes-256-cbc', SALT);
  var crypted = cipher.update(text, 'utf8', 'hex');
  crypted += cipher.final('hex');
  return crypted;
}
function decrypt(text){
  var decipher = crypto.createDecipher('aes-256-cbc', SALT);
  var dec = decipher.update(text, 'hex', 'utf8');
  dec += decipher.final('utf8');
  return dec;
}

function createToken(twoFactor, callback) {
  var headers = {};

  if (twoFactor) {
    headers['X-GitHub-OTP'] = twoFactor;
  }

  github.authorization.create({
    scopes: ['public_repo', 'repo'],
    note: 'generator-github-repository ',
    'note_url': 'https://github.com/Jimdo/generator-github-repository',
    headers: headers
  }, callback);
}

function authWithToken(token) {
  github.authenticate({
    type: 'oauth',
    token: token
  });
}

module.exports = generator.Base.extend({
  prompting: {
    username: function () {
      var done = this.async();
      var token = this._globalConfig.get('token');

      var prompt = function() {
        this.prompt([{
          type: 'input',
          name: 'username',
          message: 'What is your username on GitHub?',
          store: true
        }, {
          type: 'password',
          name: 'password',
          message: 'Please enter your GitHub Password. (wont be stored)'
        }],
        function (answers) {
          this.username = answers.username;
          this.password = answers.password;

          done();
        }.bind(this));
      }.bind(this);

      if (token) {
        authWithToken(decrypt(token));
        this.username = this._globalConfig.get('promptValues').username;

        github.user.getTeams({}, function(err, data) {
          if (err) {
            this.log(chalk.yellow('Token seems to be revoked, re-authenticating...'));
            prompt();
          } else {
            this.teams = data;
            this.authenticated = true;
          }
        }.bind(this));
      } else {
        prompt();
      }
    },
    authenticate: function() {
      if (this.authenticated) {
        return;
      }

      var self = this;
      var done = this.async();

      github.authenticate({
        type: 'basic',
        username: this.username,
        password: this.password
      });

      function storeToken(token) {
        self._globalConfig.set('token', encrypt(token));
        self._globalConfig.save();
        authWithToken(token);
        done();
      }

      createToken(null, function(err, res) {
        if (err && err.code === 401) {
          this.prompt([{
            type: 'password',
            name: 'twoFactor',
            message: 'Please enter a two-factor authentication code.'
          }],
          function (answers) {
            createToken(answers.twoFactor, function(err, res) {
              if (err) {
                done(err);
              }
              storeToken(res.token);
            });
          }.bind(this));
        } else if (err) {
          done(err);
        } else {
          storeToken(res.token);
        }
      }.bind(this));
    },
    orgs: function() {
      var done = this.async();


      var processTeams = function(data) {
        var orgs = {};
        var adminTeams = [];
        data.forEach(function(team) {
          if (team.permission === 'admin') {
            adminTeams.push(team);
            if (!orgs[team.organization.login]) {
              orgs[team.organization.login] = team.organization;
            }
          }
        });

        var users = [
          this.username
        ];

        this.useOrg = false;

        if (!adminTeams.length) {
          this.publishUnder = this.username;
          return;
        }

        this.orgs = orgs;
        _.forEach(orgs, function(org) {
          users.push(org.login);
        });

        this.prompt([{
            type: 'list',
            name: 'publishUnder',
            message: 'Who shall be the owner of this project?',
            choices: users
          }, {
            type: 'list',
            name: 'publishUnderTeam',
            message: 'Which team?',
            choices: function(answers) {
              return adminTeams.filter(function(team) {
                return team.organization.login === answers.publishUnder;
              }).map(function(team) {
                return team.name;
              });
            },
            when: function(answers) {
              return answers.publishUnder !== users[0];
            }
          }],
          function (answers) {
            this.publishUnder = answers.publishUnder;
            if (this.publishUnder !== this.username) {
              this.publishUnderTeam = adminTeams.filter(function(team) {
                return team.name === answers.publishUnderTeam;
              })[0].id;
              this.useOrg = true;
            }
            done();
          }.bind(this)
        );
      }.bind(this);

      if (!this.teams) {
        github.user.getTeams({}, function(err, data) {
          if (err) {
            done(err);
          }
          processTeams(data);
        });
      } else {
        processTeams(this.teams);
      }
    },
    additionalData: function() {
      var done = this.async();

      this.prompt([{
        type: 'input',
        name: 'reponame',
        message: 'What name should be used for the repository?',
        default: _.kebabCase(this.appname)
      }, {
        type: 'input',
        name: 'description',
        message: 'Please describe your project.',
        default: 'A project that does great things'
      }, {
        type: 'confirm',
        name: 'private',
        message: 'Is this a private project?',
        default: false,
        store: true
      }, {
        type: 'list',
        name: 'homepageType',
        message: 'Where should I point to when asked about the homepage?',
        choices: ['Repository', 'GitHub Page', 'Custom'],
        default: 0
      }, {
        type: 'input',
        name: 'homepage',
        message: 'What will be the homepage of this project?',
        default: 'http://example.org',
        when: function(answers) {
          return answers.homepageType === 'Custom';
        }
      }],
      function (answers) {
        this.reponame = answers.reponame;
        this.description = answers.description;
        this.private = answers.private;
        this.homepageType = answers.homepageType;

        if (this.homepageType === 'Repository') {
          this.homepage = 'https://github.com/' + this.publishUnder + '/' + this.reponame;
        } else if (this.homepageType === 'GitHub Page') {
          this.homepage = [
            'http://',
            this.publishUnder.toLowerCase(),
            '.github.io/',
            this.reponame,
            '/'
          ].join('');
        } else if (this.homepageType === 'Custom') {
          this.homepage = this.homepage;
        }

        done();
      }.bind(this));
    }
  },

  writing: {
    createRepo: function() {
      var done = this.async();
      var method = this.useOrg ? github.repos.createFromOrg : github.repos.create;
      var params = {
        name: this.reponame,
        description: this.description,
        homepage: this.homepage,
        private: this.private
      };
      if (this.useOrg) {
        params.org = this.publishUnder;
        // jshint camelcase: false
        params.team_id = this.publishUnderTeam;
        // jshint camelcase: true
      }

      method(params, function(err, data) {
        if (err) {
          return done(err);
        }

        var store = {
          owner: this.publishUnder
        };

        [
          'git_url',
          'html_url',
          'ssh_url',
          'homepage',
          'full_name',
          'name',
          'description'
        ].forEach(function(key) {
          store[_.camelCase(key)] = data[key];
        });

        this.config.set(store);
        this.config.save();

        done(err);

        this.log('Created ' + store.htmlUrl + ' ' + chalk.green('✔'));
      }.bind(this));
    }
  },

  install: function () {
  }
});
